"""Module to handling typing using pynput, xdotool, ydotool or wtype

"""
# flake8: noqa
import re
import time
from subprocess import call

from bwm import CONF, SEQUENCE
from bwm.menu import dmenu_err
from bwm.totp import gen_otp


def autotype_seq(entry):
    """Return value for autotype sequence

        Args: entry - dict
        Return: string

    """
    return next((i.get('value') for i in entry['fields'] if i.get('name') == 'autotype'), "")


def autotype_index(entry):
    """Returns index of the autotype field list in entry['fields']

        Args: entry - dict
        Returns: int  entry['fields'][autotype_index(entry] = \
                {"name": "autotype", "value": "{USERNAME}{TAB}{PASSWORD}{ENTER}"}

    """
    return next((entry['fields'].index(i) for i in entry['fields'] if
                 i.get('name') == 'autotype'))


def tokenize_autotype(autotype):
    """Process the autotype sequence

    Args: autotype - string
    Returns: tokens - generator ((token, if_special_char T/F), ...)

    """
    while autotype:
        opening_idx = -1
        for char in "{+^%~@":
            idx = autotype.find(char)
            if idx != -1 and (opening_idx == -1 or idx < opening_idx):
                opening_idx = idx

        if opening_idx == -1:
            # found the end of the string without further opening braces or
            # other characters
            yield autotype, False
            return

        if opening_idx > 0:
            yield autotype[:opening_idx], False

        if autotype[opening_idx] in "+^%~@":
            yield autotype[opening_idx], True
            autotype = autotype[opening_idx + 1:]
            continue

        closing_idx = autotype.find('}')
        if closing_idx == -1:
            dmenu_err("Unable to find matching right brace (}) while" +
                      f"tokenizing auto-type string: {autotype}\n")
            return
        if closing_idx == opening_idx + 1 and closing_idx + 1 < len(autotype) \
                and autotype[closing_idx + 1] == '}':
            yield "{}}", True
            autotype = autotype[closing_idx + 2:]
            continue
        yield autotype[opening_idx:closing_idx + 1], True
        autotype = autotype[closing_idx + 1:]


def token_command(token):
    """When token denotes a special command, this function provides a callable
    implementing its behaviour.

    """
    cmd = None

    def _check_delay():
        match = re.match(r'{DELAY (\d+)}', token)
        if match:
            delay = match.group(1)
            nonlocal cmd
            cmd = lambda t=delay: time.sleep(int(t) / 1000)
            return True
        return False

    if _check_delay():  # {DELAY x}
        return cmd
    return None


PLACEHOLDER_AUTOTYPE_TOKENS = {
    "{TITLE}"   : lambda e: e['name'],
    "{USERNAME}": lambda e: e['login']['username'],
    "{URL}"     : lambda e: e.get("login", "").get("uris", [])[0].get("uri", "")
                            if e.get("login", "") and e.get("login", "").get("uris", []) else "",
    "{PASSWORD}": lambda e: e['login']['password'],
    "{NOTES}"   : lambda e: e['notes'],
    "{CARDNUM}" : lambda e: e['card']['number'],
    "{TOTP}"    : lambda e: gen_otp(e['login']['totp']),
}

STRING_AUTOTYPE_TOKENS = {
    "{PLUS}"      : '+',
    "{PERCENT}"   : '%',
    "{CARET}"     : '^',
    "{TILDE}"     : '~',
    "{LEFTPAREN}" : '(',
    "{RIGHTPAREN}": ')',
    "{LEFTBRACE}" : '{',
    "{RIGHTBRACE}": '}',
    "{AT}"        : '@',
    "{+}"         : '+',
    "{%}"         : '%',
    "{^}"         : '^',
    "{~}"         : '~',
    "{(}"         : '(',
    "{)}"         : ')',
    "{[}"         : '[',
    "{]}"         : ']',
    "{{}"         : '{',
    "{}}"         : '}',
}


def type_entry_pynput(entry, tokens):  # pylint: disable=too-many-branches
    """Use pynput to auto-type the selected entry

    """
    try:
        from pynput import keyboard
        from .tokens_pynput import AUTOTYPE_TOKENS
    except ModuleNotFoundError:
        return
    kbd = keyboard.Controller()
    enter_idx = True
    for token, special in tokens:
        if special:
            cmd = token_command(token)
            if callable(cmd):
                cmd()  # pylint: disable=not-callable
            elif token in PLACEHOLDER_AUTOTYPE_TOKENS:
                to_type = PLACEHOLDER_AUTOTYPE_TOKENS[token](entry)
                if to_type:
                    try:
                        kbd.type(to_type)
                    except kbd.InvalidCharacterException:
                        dmenu_err("Unable to type string...bad character.\n"
                                  "Try setting `type_library = xdotool` in config.ini")
                        return
            elif token in STRING_AUTOTYPE_TOKENS:
                to_type = STRING_AUTOTYPE_TOKENS[token]
                try:
                    kbd.type(to_type)
                except kbd.InvalidCharacterException:
                    dmenu_err("Unable to type string...bad character.\n"
                              "Try setting `type_library = xdotool` in config.ini")
                    return
            elif token in AUTOTYPE_TOKENS:
                to_tap = AUTOTYPE_TOKENS[token]
                kbd.tap(to_tap)
                # Add extra {ENTER} key tap for first instance of {ENTER}. It
                # doesn't get recognized for some reason.
                if enter_idx is True and token in ("{ENTER}", "~"):
                    kbd.tap(to_tap)
                    enter_idx = False
            else:
                dmenu_err(f"Unsupported auto-type token (pynput): {token}")
                return
        else:
            try:
                kbd.type(token)
            except kbd.InvalidCharacterException:
                dmenu_err("Unable to type string...bad character.\n"
                          "Try setting `type_library = xdotool` in config.ini")
                return


def type_entry_xdotool(entry, tokens):
    """Auto-type entry entry using xdotool

    """
    enter_idx = True
    from .tokens_xdotool import AUTOTYPE_TOKENS
    for token, special in tokens:
        if special:
            cmd = token_command(token)
            if callable(cmd):
                cmd()  # pylint: disable=not-callable
            elif token in PLACEHOLDER_AUTOTYPE_TOKENS:
                to_type = PLACEHOLDER_AUTOTYPE_TOKENS[token](entry)
                if to_type:
                    call(['xdotool', 'type', to_type])
            elif token in STRING_AUTOTYPE_TOKENS:
                to_type = STRING_AUTOTYPE_TOKENS[token]
                call(['xdotool', 'type', to_type])
            elif token in AUTOTYPE_TOKENS:
                cmd = ['xdotool'] + AUTOTYPE_TOKENS[token]
                call(cmd)
                # Add extra {ENTER} key tap for first instance of {ENTER}. It
                # doesn't get recognized for some reason.
                if enter_idx is True and token in ("{ENTER}", "~"):
                    cmd = ['xdotool'] + AUTOTYPE_TOKENS[token]
                    call(cmd)
                    enter_idx = False
            else:
                dmenu_err(f"Unsupported auto-type token (xdotool): {token}")
                return
        else:
            call(['xdotool', 'type', token])


def type_entry_ydotool(entry, tokens):
    """Auto-type entry entry using ydotool

    """
    enter_idx = True
    from .tokens_ydotool import AUTOTYPE_TOKENS
    for token, special in tokens:
        if special:
            cmd = token_command(token)
            if callable(cmd):
                cmd()  # pylint: disable=not-callable
            elif token in PLACEHOLDER_AUTOTYPE_TOKENS:
                to_type = PLACEHOLDER_AUTOTYPE_TOKENS[token](entry)
                if to_type:
                    call(['ydotool', 'type', to_type])
            elif token in STRING_AUTOTYPE_TOKENS:
                to_type = STRING_AUTOTYPE_TOKENS[token]
                call(['ydotool', 'type', to_type])
            elif token in AUTOTYPE_TOKENS:
                cmd = ['ydotool'] + AUTOTYPE_TOKENS[token]
                call(cmd)
                # Add extra {ENTER} key tap for first instance of {ENTER}. It
                # doesn't get recognized for some reason.
                if enter_idx is True and token in ("{ENTER}", "~"):
                    cmd = ['ydotool'] + AUTOTYPE_TOKENS[token]
                    call(cmd)
                    enter_idx = False
            else:
                dmenu_err(f"Unsupported auto-type token (ydotool): {token}")
                return
        else:
            call(['ydotool', 'type', token])


def type_entry_wtype(entry, tokens):
    """Auto-type entry entry using wtype

    """
    from .tokens_wtype import AUTOTYPE_TOKENS
    for token, special in tokens:
        if special:
            cmd = token_command(token)
            if callable(cmd):
                cmd()  # pylint: disable=not-callable
            elif token in PLACEHOLDER_AUTOTYPE_TOKENS:
                to_type = PLACEHOLDER_AUTOTYPE_TOKENS[token](entry)
                if to_type:
                    call(['wtype', '--', to_type])
            elif token in STRING_AUTOTYPE_TOKENS:
                to_type = STRING_AUTOTYPE_TOKENS[token]
                call(['wtype', '--', to_type])
            elif token in AUTOTYPE_TOKENS:
                cmd = ['wtype', '-k', AUTOTYPE_TOKENS[token]]
                call(cmd)
            else:
                dmenu_err(f"Unsupported auto-type token (wtype): {token}")
                return
        else:
            call(['wtype', '--', token])


def type_entry(entry, atype=""):
    """Pick which library to use to type strings

    Defaults to pynput

    Args: entry - dict

    """
    # Don't autotype anything except for login and cards - for now TODO
    if entry['type'] not in (1, 3):
        dmenu_err("Autotype currently disabled for this type of entry")
        return
    # Autotype for entry > CLI --autotype > config.ini autotype
    sequence = autotype_seq(entry)
    if sequence == 'False':
        dmenu_err("Autotype disabled for this entry")
        return
    if not sequence or sequence == 'None':
        sequence = atype
        if entry['type'] == 3:
            sequence = "{CARDNUM}"
    tokens = tokenize_autotype(sequence)

    library = 'pynput'
    if CONF.has_option('vault', 'type_library'):
        library = CONF.get('vault', 'type_library')
    if library == 'xdotool':
        type_entry_xdotool(entry, tokens)
    elif library == 'ydotool':
        type_entry_ydotool(entry, tokens)
    elif library == 'wtype':
        type_entry_wtype(entry, tokens)
    else:
        type_entry_pynput(entry, tokens)


def type_text(data):
    """Type the given text data

    """
    library = 'pynput'
    if CONF.has_option('vault', 'type_library'):
        library = CONF.get('vault', 'type_library')
    if library == 'xdotool':
        call(['xdotool', 'type', data])
    elif library == 'ydotool':
        call(['ydotool', 'type', data])
    elif library == 'wtype':
        call(['wtype', '--', data])
    else:
        try:
            from pynput import keyboard
        except ModuleNotFoundError:
            return
        kbd = keyboard.Controller()
        try:
            kbd.type(data)
        except kbd.InvalidCharacterException:
            dmenu_err("Unable to type string...bad character.\n"
                      "Try setting `type_library = xdotool` in config.ini")

# vim: set et ts=4 sw=4 :



"""Read, type and edit Bitwarden vault entries using dmenu style launchers

"""
import argparse
from contextlib import closing
import multiprocessing
from multiprocessing.managers import BaseManager
import os
from os.path import exists, expanduser
import random
import socket
import string
from subprocess import call

import bwm
from bwm.bwm import DmenuRunner
from bwm.menu import dmenu_err


def find_free_port():
    """Find random free port to use for BaseManager server

    Returns: int Port

    """
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        sock.bind(('127.0.0.1', 0))  # pylint:disable=no-member
        return sock.getsockname()[1]  # pylint:disable=no-member


def random_str():
    """Generate random auth string for BaseManager

    Returns: string

    """
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for i in range(15))


def get_auth():
    """Generate and save port and authkey to ~/.cache/.bwm-auth

    Returns: int port, bytestring authkey

    """
    auth = bwm.configparser.ConfigParser()
    if not exists(bwm.AUTH_FILE):
        fdr = os.open(bwm.AUTH_FILE, os.O_WRONLY | os.O_CREAT, 0o600)
        with open(fdr, 'w', encoding=bwm.ENC) as a_file:
            auth.set('DEFAULT', 'port', str(find_free_port()))
            auth.set('DEFAULT', 'authkey', random_str())
            auth.write(a_file)
    try:
        auth.read(bwm.AUTH_FILE)
        port = auth.get('DEFAULT', 'port')
        authkey = auth.get('DEFAULT', 'authkey').encode()
    except (bwm.configparser.NoOptionError,
            bwm.configparser.MissingSectionHeaderError,
            bwm.configparser.ParsingError,
            multiprocessing.context.AuthenticationError):
        os.remove(bwm.AUTH_FILE)
        dmenu_err("Cache file was corrupted. Stopping all instances. Please try again")
        call(["pkill", "bwm"])  # Kill all prior instances as well
        return None, None
    return int(port), authkey


def client():
    """Define client connection to server BaseManager

    Returns: BaseManager object
    """
    port, auth = get_auth()
    mgr = BaseManager(address=('', port), authkey=auth)
    mgr.register('set_event')
    mgr.register('get_pipe')
    mgr.register('read_args_from_pipe')
    mgr.connect()
    return mgr


class Server(multiprocessing.Process):  # pylint: disable=too-many-instance-attributes
    """Run BaseManager server to listen for dmenu calling events

    """
    def __init__(self):
        multiprocessing.Process.__init__(self)
        self.port, self.authkey = get_auth()
        self.start_flag = multiprocessing.Event()
        self.kill_flag = multiprocessing.Event()
        self.cache_time_expired = multiprocessing.Event()
        self.args_flag = multiprocessing.Event()
        self.start_flag.set()
        self.args = None
        self._parent_conn, self._child_conn = multiprocessing.Pipe(duplex=False)

    def run(self):
        _ = self.server()
        self.kill_flag.wait()

    def _get_pipe(self):
        return self._child_conn

    def get_args(self):
        """ Reads arguments sent by the client to the server

        """
        return self._parent_conn.recv()

    def server(self):
        """Set up BaseManager server

        """
        mgr = BaseManager(address=('127.0.0.1', self.port),
                          authkey=self.authkey)
        mgr.register('set_event', callable=self.start_flag.set)
        mgr.register('get_pipe', callable=self._get_pipe)
        mgr.register('read_args_from_pipe', callable=self.args_flag.set)
        mgr.start()  # pylint: disable=consider-using-with
        return mgr


def run(**kwargs):
    """Main entrypoint. Start the background Manager and Dmenu runner processes.

    """
    server = Server()
    dmenu = DmenuRunner(server, **kwargs)
    dmenu.daemon = True
    server.start()
    dmenu.start()
    server.join()
    if exists(expanduser(bwm.AUTH_FILE)):
        os.remove(expanduser(bwm.AUTH_FILE))


def main():
    """Main script entrypoint

    """
    parser = argparse.ArgumentParser(
        description="Dmenu-compatible launcher frontend for Bitwarden/Vaultwarden")

    parser.add_argument(
        "-a",
        "--autotype",
        type=str,
        required=False,
        help="Override autotype sequence in config.ini",
    )

    parser.add_argument(
        "-l",
        "--login",
        type=str,
        required=False,
        help="Login email address",
    )

    parser.add_argument(
        "-v",
        "--vault",
        type=str,
        required=False,
        help="Vault URL to open, skipping the database selection menu",
    )

    args = vars(parser.parse_args())

    args = args if any(args.values()) else {}

    try:
        manager = client()
        conn = manager.get_pipe()  # pylint: disable=no-member
        if args:
            conn.send(args)
            manager.read_args_from_pipe()  # pylint: disable=no-member
        manager.set_event()  # pylint: disable=no-member
    except ConnectionRefusedError:
        run(**args)


if __name__ == '__main__':
    main()

# vim: set et ts=4 sw=4 :



"""Bitwarden-menu view functions

"""
from os.path import join
import webbrowser

from bwm.menu import dmenu_select
from bwm.totp import gen_otp
import bwm


def obj_name(obj, oid):
    """Return name of folder/collection object based on id

        Args: obj - dict
              oid - string

    """
    path = obj[oid]['name']
    if path == 'No Folder':
        path = '/'
    return path


def view_all_entries(options, vault_entries, folders):
    """Generate numbered list of all vault entries and open with dmenu.

    Returns: dmenu selection

    """
    num_align = len(str(len(vault_entries)))
    # Login: Num(l) - Folder/name - username - url
    bw_login_pattern = str("{:>{na}}(l) - {} - {} - {}")
    # Secure Note: Num(n) - Folder/name
    bw_note_pattern = str("{:>{na}}(n) - {}")
    # Card: Num(c) - Folder/name - card type - card owner - card #
    bw_card_pattern = str("{:>{na}}(c) - {} - {} - {} - {}")
    # Identity: Num(i) - Folder/name - lastName, firstName - email - phone
    bw_ident_pattern = str("{:>{na}}(i) - {} - {}, {} - {} - {}")
    # Have to number each entry to capture duplicates correctly
    ven = []
    for j, i in enumerate(vault_entries):
        if i['type'] == 1:
            ven.append(bw_login_pattern.format(
                j,
                join(obj_name(folders, i['folderId']), i['name']),
                i['login']['username'],
                make_url_entries(i)[0].split(": ", 1)[1],
                na=num_align))
        elif i['type'] == 2:
            ven.append(bw_note_pattern.format(
                j,
                join(obj_name(folders, i['folderId']), i['name']),
                na=num_align))
        elif i['type'] == 3:
            ven.append(bw_card_pattern.format(
                j,
                join(obj_name(folders, i['folderId']), i['name']),
                i['card']['brand'],
                i['card']['cardholderName'],
                i['card']['number'],
                na=num_align))
        elif i['type'] == 4:
            ven.append(bw_ident_pattern.format(
                j,
                join(obj_name(folders, i['folderId']), i['name']),
                i['identity']['lastName'],
                i['identity']['firstName'],
                i['identity']['email'],
                i['identity']['phone'],
                na=num_align))
    vault_entries_s = str("\n").join(ven)
    if options:
        options_s = ("\n".join(options) + "\n")
        entries_s = options_s + vault_entries_s
    else:
        entries_s = vault_entries_s
    return dmenu_select(min(bwm.MAX_LEN, len(options) + len(vault_entries)), inp=entries_s)


def view_entry(entry, folders):
    """Show an entry (login, card, identity or secure note)

    """
    entry_types = {1: view_login, 2: view_note, 3: view_card, 4: view_ident}
    return entry_types[entry['type']](entry, folders)


def make_url_entries(entry):
    """Parse multiple URL's for viewing

    Args: entry
    Returns: list of strings ["URL: xxxxx", "URL1: xxxx", "URL2: xxxx"]

    """
    urls = entry.get('login').get('uris') if entry.get('login') else None
    return [f"URL{j}: {i['uri']}" for j, i in enumerate(urls, 1)] if urls else ["URL: None"]


def view_login(entry, folders):
    """Show title, username, password, url and notes for a login entry.

    Returns: dmenu selection to type

    """
    fields = [f"Title: {entry['name'] or 'None'}",
              f"Folder: {obj_name(folders, entry['folderId'])}",
              f"Username: {entry['login']['username'] or 'None'}",
              f"Password: {'**********' if entry['login']['password'] else 'None'}",
              f"TOTP: {'******' if entry['login']['totp'] else 'None'}",
              f"Notes: {'<Enter to view>' if entry['notes'] else 'None'}"]
    fields[-1:-1] = make_url_entries(entry)
    sel = dmenu_select(len(fields), inp="\n".join(fields))
    if sel.endswith(": None") or sel not in fields:
        return ""
    if sel == "Notes: <Enter to view>":
        sel = view_notes(entry['notes'])
    elif sel == 'Password: **********':
        sel = entry['login']['password']
    elif sel == "TOTP: ******":
        sel = gen_otp(entry['login']['totp'])
    elif sel.startswith("URL"):
        if sel != "URL: None":
            webbrowser.open(sel.split(": ", 1)[-1])
        sel = ""
    else:
        sel = sel.split(": ", 1)[1]
    return sel


def view_note(entry, folders):
    """Show title and note for a secure note entry.

    Returns: dmenu selection

    """
    fields = [f"Title: {entry['name'] or 'None'}",
              f"Folder: {obj_name(folders, entry['folderId'])}",
              f"Notes: {'<Enter to view>' if entry['notes'] else 'None'}"]
    sel = dmenu_select(len(fields), inp="\n".join(fields))
    if sel.endswith(": None") or sel not in fields:
        return ""
    if sel == "Notes: <Enter to view>":
        sel = view_notes(entry['notes'])
    else:
        sel = sel.split(": ", 1)[1]
    return sel


def view_card(entry, folders):
    """Show title, card info and notes for a card entry.

    Returns: dmenu selection

    """
    fields = [f"Title: {entry['name'] or 'None'}",
              f"Folder: {obj_name(folders, entry['folderId'])}",
              f"Notes: {'<Enter to view>' if entry['notes'] else 'None'}"]
    fields[-1:-1] = [f"{i}: {entry['card'][j] or 'None'}" for i, j in bwm.CARD.items()]
    sel = dmenu_select(len(fields), inp="\n".join(fields))
    if sel.endswith(": None") or sel not in fields:
        return ""
    if sel == "Notes: <Enter to view>":
        sel = view_notes(entry['notes'])
    else:
        sel = sel.split(": ", 1)[1]
    return sel


def view_ident(entry, folders):
    """Show title, identify information and notes for an identity entry.

    Returns: dmenu selection

    """
    fields = [f"Title: {entry['name'] or 'None'}",
              f"Folder: {obj_name(folders, entry['folderId'])}",
              f"Notes: {'<Enter to view>' if entry['notes'] else 'None'}"]
    fields[-1:-1] = [f"{i}: {entry['identity'][j] or 'None'}" for i, j in bwm.IDENTITY.items()]
    sel = dmenu_select(len(fields), inp="\n".join(fields))
    if sel.endswith(": None") or sel not in fields:
        return ""
    if sel == "Notes: <Enter to view>":
        sel = view_notes(entry['notes'])
    else:
        sel = sel.split(": ", 1)[1]
    return sel


def view_notes(notes):
    """View the 'Notes' field line-by-line within dmenu.

    Returns: text of the selected line for typing

    """
    sel = dmenu_select(min(bwm.MAX_LEN, len(notes.split('\n'))), inp=notes)
    return sel



"""Launcher functions

"""
import shlex
import sys
from subprocess import run

import bwm


def dmenu_cmd(num_lines, prompt):
    """Parse config.ini for dmenu options

    Args: args - num_lines: number of lines to display
                 prompt: prompt to show
    Returns: command invocation (as a list of strings) for
                ["dmenu", "-l", "<num_lines>", "-p", "<prompt>", "-i", ...]

    """
    commands = {"bemenu": ["-p", str(prompt), "-l", str(num_lines)],
                "dmenu": ["-p", str(prompt), "-l", str(num_lines)],
                "rofi": ["-dmenu", "-p", str(prompt), "-l", str(num_lines)],
                "wofi": ["-p", str(prompt), "-L", str(num_lines)]}
    command = shlex.split(bwm.CONF.get('dmenu', 'dmenu_command', fallback='dmenu'))
    command.extend(commands.get(command[0], []))
    pwprompts = ("Password", "password", "client_secret", "Verify password", "Enter Password")
    obscure = bwm.CONF.getboolean('dmenu_passphrase', 'obscure', fallback=True)
    if any(i == prompt for i in pwprompts) and obscure is True:
        pass_prompts = {"dmenu": dmenu_pass(command[0]),
                        "rofi": ['-password'],
                        "bemenu": ['-x'],
                        "wofi": ['-P']}
        command.extend(pass_prompts.get(command[0], []))
    return command


def dmenu_pass(command):
    """Check if dmenu passphrase patch is applied and return the correct command
    line arg list

    Args: command - string
    Returns: list or None

    """
    if command != 'dmenu':
        return None
    try:
        # Check for dmenu password patch
        dm_patch = b'P' in run(["dmenu", "-h"],
                               capture_output=True,
                               check=False).stderr
    except FileNotFoundError:
        dm_patch = False
    color = bwm.CONF.get('dmenu_passphrase', 'obscure_color', fallback="#222222")
    return ["-P"] if dm_patch else ["-nb", color, "-nf", color]


def dmenu_select(num_lines, prompt="Entries", inp=""):
    """Call dmenu and return the selected entry

    Args: num_lines - number of lines to display
          prompt - prompt to show
          inp - string to pass to dmenu via STDIN

    Returns: sel - string

    """
    cmd = dmenu_cmd(num_lines, prompt)
    res = run(cmd,
              capture_output=True,
              check=False,
              input=inp,
              encoding=bwm.ENC,
              env=bwm.ENV)
    if res.stderr:
        print(res.stderr)
        sys.exit()
    return res.stdout.rstrip('\n') if res.stdout is not None else None


def dmenu_err(prompt):
    """Pops up a dmenu prompt with an error message

    """
    try:
        prompt = prompt.decode(bwm.ENC)
    except AttributeError:
        pass
    return dmenu_select(len(prompt.splitlines()), "Error", inp=prompt)

# vim: set et ts=4 sw=4 :



#!/usr/bin/env python3  # pylint: disable=missing-docstring

from setuptools import setup

setup(name="bitwarden-menu",
      version="v0.4.1",
      description="Dmenu/Rofi frontend for Bitwarden CLI tool",
      long_description=open('README.md', 'rb').read().decode('utf-8'),
      long_description_content_type="text/markdown",
      author="Scott Hansen",
      author_email="firecat4153@gmail.com",
      url="https://github.com/firecat53/bitwarden-menu",
      download_url="https://github.com/firecat53/bitwarden-menu/tarball/v0.4.1",
      packages=['bwm'],
      entry_points={
          'console_scripts': ['bwm=bwm.__main__:main']
      },
      data_files=[('share/doc/bitwarden-menu', ['README.md', 'LICENSE',
                                                'config.ini.example']),
                  ('share/doc/bitwarden-menu/docs', ['docs/install.md',
                                                     'docs/configure.md',
                                                     'docs/usage.md']),
                  ('share/man/man1', ['bwm.1'])],
      install_requires=["pynput", "xdg"],
      license="MIT",
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Topic :: Utilities',
      ],
      keywords=("dmenu rofi bitwarden bitwarden-menu bwm"),
      )
